//  Java CGI -- Classes to aid in writing CGI programs in Java.
//
//  Copyright (c) 1996, David H. Silber  (dhs@orbits.com)
//
//    My time to write this free software donated by:
//                   Stellar Orbits Technology Services
//              http://www.orbits.com/   or   info@orbits.com
//
//    I can be contacted at:    1923 Rockland Avenue
//                              Rockville, MD  20851
//
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Library General Public
//    License version 2 as published by the Free Software Foundation.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Library General Public License for more details.
//
//    You should have received a copy of the GNU Library General Public
//    License along with this library; if not, write to the Free
//    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

package Orbits.net;

import java.io.*;
import java.util.*;

public class CGI {

	//  Environment variables are the default list of values.
	private Properties nameValues = new Properties ();


	//  Constructor.
	public CGI() {

		//  Get the name of the file which contains the data provided
		//    by the HTTP daemon and parse that information into the
		//    local storage (nameValues).
		parseDaemonInput( System.getProperty( "DataID" ) );

		//  Get any information available on the standard input stream.
		parsePostInput();

		//  Get any information encoded into `QUERY_STRING'.  This
		//    variable is part of the data provided by the HTTP daemon.
		parseGetInput();
	}


	//  Let users of this class get values out by specifying a name.
	public String getValue ( String name ) {
		return ( nameValues.getProperty( name ) );
	}


	//  Let users of this class have access to the list of names.
	public Enumeration getNames () {
		return ( nameValues.propertyNames() );
	}


	//  Translate hex digits into the character that they represent.
	char hexValue( int digit1, int digit2 ) {
		int radix = 16;

		int value = Character.digit( (char)digit1, radix ) * radix;
		value += Character.digit( (char)digit2, radix );

		return ( (char)value );
	}


	//  Parse through the input, matching data names with data values in
	//    the nameValue Properties instance.  Name/value pairs are grouped
	//    between ampersands (`&') and the start of line & end of line.
	//    Names and their associated values are separated by an equals
	//    sign (`=').  Plus signs (`+') are turned into spaces.  A percent
	//    sign (`%') turns the following two digits into a character.
	void parseCgiData( InputStream cgiData ) throws IOException {
		int c;
		StringBuffer buffy = new StringBuffer();
		String name = new String();

		//  Get each character in sequence.
		while ((c = cgiData.read()) > -1) {

			//  The default case adds characters to an
			//    accumulation buffer.  The other cases
			//    provide special processing for special
			//    needs.
			switch ( c ) {

				case '=' :
					//  Move accumulation buffer to name.
					name = buffy.toString();
					buffy.setLength( 0 );
					break;

				case '&' :
				case '\n' :
					//  Move accumulation buffer to value.
					nameValues.put( name, buffy.toString());
					buffy.setLength( 0 );
					break;

				case '+' :
					//  A plus turns into a space.
					buffy.append( " " );
					break;

				case '%' :
					//  Grab the next two characters, turn
					//    them into a hexidecimal number,
					//    and turn that into a character.
					int digit1 = cgiData.read();
					int digit2 = cgiData.read();

					buffy.append(
						hexValue( digit1, digit2 ) );
					break;

				default :
					//  Add c to accumulation buf.
					buffy.append( (char)c );
					break;
			}
		}
		//  Move accumulation buffer to value.
		nameValues.put( name, buffy.toString() );
	}


	//  Parse through the input, matching data names with data values in
	//    the nameValue Properties instance.  All data after the initial
	//    equals sign (`=') is part of the value, even if it includes an
	//    equals sign.
	void parseDaemonData( InputStream daemonData ) throws IOException {
		int c;
		StringBuffer buffy = new StringBuffer();
		String name = new String();

		//  This is true while we are getting the value of a variable
		//    (and therefore ignoring any special meaning for `=').
		boolean inValue = false;

		//  Get each character in sequence.
		while ((c = daemonData.read()) > -1) {

			//  The default case adds characters to an
			//    accumulation buffer.  The other cases
			//    provide special processing for special
			//    needs.
			switch ( c ) {

				case '=' :
					if (!inValue) {
						//  Move accumulation buffer
						//    to name.
						name = buffy.toString();
						buffy.setLength( 0 );
						inValue = true;
					}
					else {
						//  Add c to accumulation buf.
						buffy.append( (char)c );
					}
					break;

				case '\n' :
					//  Move accumulation buffer to value.
					nameValues.put( name, buffy.toString());
					buffy.setLength( 0 );
					inValue = false;
					break;

				default :
					//  Add c to accumulation buf.
					buffy.append( (char)c );
					break;
			}
		}
		//  Move accumulation buffer to value.
		nameValues.put( name, buffy.toString() );
	}


	//  This method gets CGI-formatted data from the environment variable
	//    ``QUERY_STRING''.  The ``QUERY_STRING'' data must have already
	//    been loaded in via parseDaemonInput().
	void parseGetInput() {
		try {
			StringBufferInputStream cgiIn;
			cgiIn = new StringBufferInputStream(
						getValue( "QUERY_STRING" ) );

			parseCgiData ( cgiIn );
		} catch (IOException e) {
			e.printStackTrace();
		}
	}


	//  This method gets CGI-formatted data from the standard input.
	void parsePostInput() {
		try {
			parseCgiData ( System.in );
		} catch (IOException e) {
			e.printStackTrace();
		}
	}


	//  This method adds information from the HTTP daemon to the set
	//    of CGI information.
	void parseDaemonInput( String daemonDataName ) {
		try {
			FileInputStream daemonData = 
					new FileInputStream( daemonDataName );
			parseDaemonData( daemonData );
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}

